import { useLayoutEffect, useState } from 'react';
import useEvent from 'react-use-event-hook';

function refsInitialState() {
  return {
    proms: []
  };
}

function useStateWithPromise(initialState) {
  const [state, setState] = useState(initialState);
  const [$refs] = useState(refsInitialState);
  useLayoutEffect(() => {
    if (!$refs.proms.length) return;
    const proms = $refs.proms.splice(0, $refs.proms.length);
    proms.forEach(prom => prom.options.alwaysResolve || prom.nextState === state ? prom.resolve(state) : prom.reject(state));
  }, [$refs, state]);
  const setStateWithPromise = useEvent((stateAction, options = {}) => {
    let prom = {
      options
    };
    const ret = new Promise((resolve, reject) => {
      prom.resolve = resolve;
      prom.reject = reject;
    });

    if (typeof stateAction === 'function') {
      setState(prevState => {
        prom.nextState = stateAction(prevState);
        if (prevState === prom.nextState) prom.resolve(prevState);else $refs.proms.push(prom);
        return prom.nextState;
      });
    } else {
      if (stateAction === state) prom.resolve(state);else {
        $refs.proms.push(prom);
        setState(prom.nextState = stateAction);
      }
    }

    return ret;
  });
  return [state, setStateWithPromise];
}

export default useStateWithPromise;